@@ -12,7 +12,7 @@ use super::*;
 use kas::cast::Cast;
 use kas::draw::color::Rgba;
 use kas::draw::*;
-use kas::geom::{Coord, Quad, Rect, Size, Vec2};
+use kas::geom::{Quad, Rect, Size, Vec2};
 use kas::text::{Effect, TextDisplay};
 use kas_theme::DrawShadedImpl;
 
@@ -34,7 +34,7 @@ impl<C: CustomPipe> DrawPipe<C> {
             entries: &[
                 wgpu::BindGroupLayoutEntry {
                     binding: 0,
-                    visibility: wgpu::ShaderStage::VERTEX,
+                    visibility: wgpu::ShaderStages::VERTEX,
                     ty: wgpu::BindingType::Buffer {
                         ty: wgpu::BufferBindingType::Uniform,
                         has_dynamic_offset: false,
@@ -44,7 +44,7 @@ impl<C: CustomPipe> DrawPipe<C> {
                 },
                 wgpu::BindGroupLayoutEntry {
                     binding: 1,
-                    visibility: wgpu::ShaderStage::FRAGMENT,
+                    visibility: wgpu::ShaderStages::FRAGMENT,
                     ty: wgpu::BindingType::Buffer {
                         ty: wgpu::BufferBindingType::Uniform,
                         has_dynamic_offset: false,
@@ -68,13 +68,14 @@ impl<C: CustomPipe> DrawPipe<C> {
         let light_norm_buf = device.create_buffer_init(&wgpu::util::BufferInitDescriptor {
             label: Some("light_norm_buf"),
             contents: bytemuck::cast_slice(&light_norm),
-            usage: wgpu::BufferUsage::UNIFORM | wgpu::BufferUsage::COPY_DST,
+            usage: wgpu::BufferUsages::UNIFORM | wgpu::BufferUsages::COPY_DST,
         });
 
         let images = images::Images::new(&device, &shaders, &bgl_common);
         let shaded_square = shaded_square::Pipeline::new(&device, &shaders, &bgl_common);
         let shaded_round = shaded_round::Pipeline::new(&device, &shaders, &bgl_common);
         let flat_round = flat_round::Pipeline::new(&device, &shaders, &bgl_common);
+        let round_2col = round_2col::Pipeline::new(&device, &shaders, &bgl_common);
         let custom = custom.build(&device, &bgl_common, RENDER_TEX_FORMAT);
         let text = text_pipe::Pipeline::new(&device, &shaders, &bgl_common, raster_config);
 
@@ -90,52 +91,37 @@ impl<C: CustomPipe> DrawPipe<C> {
             shaded_square,
             shaded_round,
             flat_round,
+            round_2col,
             custom,
             text,
         }
     }
 
     /// Construct per-window state
-    pub fn new_window(&self, size: Size) -> DrawWindow<C::Window> {
-        let vsize = Vec2::from(size);
-        let scale: Scale = [-0.5 * vsize.0, 0.5 * vsize.1, 2.0 / vsize.0, -2.0 / vsize.1];
-
-        let rect = Rect::new(Coord::ZERO, size);
-
-        let custom = self.custom.new_window(&self.device, size);
+    pub fn new_window(&self) -> DrawWindow<C::Window> {
+        let custom = self.custom.new_window(&self.device);
 
         DrawWindow {
-            scale,
-            clip_regions: vec![(rect, Offset::ZERO)],
+            scale: Default::default(),
+            clip_regions: vec![Default::default()],
             images: Default::default(),
             shaded_square: Default::default(),
             shaded_round: Default::default(),
             flat_round: Default::default(),
+            round_2col: Default::default(),
             custom,
             text: Default::default(),
         }
     }
 
-    /// Wraps [`wgpu::Device::create_swap_chain`]
-    pub fn create_swap_chain(
-        &self,
-        surface: &wgpu::Surface,
-        desc: &wgpu::SwapChainDescriptor,
-    ) -> wgpu::SwapChain {
-        self.device.create_swap_chain(surface, desc)
-    }
-
     /// Process window resize
     pub fn resize(&self, window: &mut DrawWindow<C::Window>, size: Size) {
         window.clip_regions[0].0.size = size;
 
         let vsize = Vec2::from(size);
-        window.scale = [
-            -0.5 * vsize.0,
-            -0.5 * vsize.1,
-            2.0 / vsize.0,
-            -2.0 / vsize.1,
-        ];
+        let off = vsize * -0.5;
+        let scale = 2.0 / vsize;
+        window.scale = [off.0, off.1, scale.0, -scale.1];
 
         self.custom
             .resize(&mut window.custom, &self.device, &self.queue, size);
@@ -174,7 +160,7 @@ impl<C: CustomPipe> DrawPipe<C> {
                     let scale_buf = device.create_buffer_init(&wgpu::util::BufferInitDescriptor {
                         label: Some("scale_buf"),
                         contents: bytemuck::cast_slice(&scale),
-                        usage: wgpu::BufferUsage::UNIFORM | wgpu::BufferUsage::COPY_DST,
+                        usage: wgpu::BufferUsages::UNIFORM | wgpu::BufferUsages::COPY_DST,
                     });
                     let bg_common = device.create_bind_group(&wgpu::BindGroupDescriptor {
                         label: Some("common bind group"),
@@ -224,6 +210,9 @@ impl<C: CustomPipe> DrawPipe<C> {
         window
             .flat_round
             .write_buffers(&self.device, &mut self.staging_belt, &mut encoder);
+        window
+            .round_2col
+            .write_buffers(&self.device, &mut self.staging_belt, &mut encoder);
         self.custom.prepare(
             &mut window.custom,
             &self.device,
@@ -264,6 +253,8 @@ impl<C: CustomPipe> DrawPipe<C> {
                     rect.size.1.cast(),
                 );
 
+                self.round_2col
+                    .render(&window.round_2col, pass, &mut rpass, bg_common);
                 self.shaded_square
                     .render(&window.shaded_square, pass, &mut rpass, bg_common);
                 self.images
@@ -436,16 +427,18 @@ impl<CW: CustomWindow> DrawRoundedImpl for DrawWindow<CW> {
     }
 
     #[inline]
-    fn rounded_frame(
-        &mut self,
-        pass: PassId,
-        outer: Quad,
-        inner: Quad,
-        inner_radius: f32,
-        col: Rgba,
-    ) {
-        self.flat_round
-            .rounded_frame(pass, outer, inner, inner_radius, col);
+    fn circle_2col(&mut self, pass: PassId, rect: Quad, col1: Rgba, col2: Rgba) {
+        self.round_2col.circle(pass, rect, col1, col2);
+    }
+
+    #[inline]
+    fn rounded_frame(&mut self, pass: PassId, outer: Quad, inner: Quad, r1: f32, col: Rgba) {
+        self.flat_round.rounded_frame(pass, outer, inner, r1, col);
+    }
+
+    #[inline]
+    fn rounded_frame_2col(&mut self, pass: PassId, outer: Quad, inner: Quad, c1: Rgba, c2: Rgba) {
+        self.round_2col.frame(pass, outer, inner, c1, c2);
     }
 }
 
