@@ -12,14 +12,22 @@ use kas::geom::{Quad, Vec2};
 use std::mem::size_of;
 
 /// Offset relative to the size of a pixel used by the fragment shader to
-/// implement multi-sampling.
-const OFFSET: f32 = 0.125;
+/// implement 4x multi-sampling. The pattern is defined by the fragment shader.
+const AA_OFFSET: f32 = 0.5 * std::f32::consts::FRAC_1_SQRT_2;
 
 // NOTE(opt): in theory we could reduce data transmission to the GPU by 1/3 by
 // sending quads (two triangles) as instances in triangle-strip mode. The
 // "frame" shape could support four-triangle strips. However, this would require
 // many rpass.draw() commands or shaders unpacking vertices from instance data.
 
+/// Vertex
+///
+/// -   `screen_pos: Vec2` — screen coordinate
+/// -   `colour: Rgba`
+/// -   `inner: f32` — inner radius, relative to outer (range: 0 to 1)
+/// -   `circle_pos: Vec2` — coordinate on virtual circle with radius 1 centred
+///     on the origin
+/// -   `pix_offset: Vec2` — size of a pixel on the virtual circle; used for AA
 #[repr(C)]
 #[derive(Clone, Copy, Debug)]
 pub struct Vertex(Vec2, Rgba, f32, Vec2, Vec2);
@@ -35,6 +43,8 @@ impl Vertex {
 pub type Window = common::Window<Vertex>;
 
 /// A pipeline for rendering rounded shapes
+///
+/// Uses 4x sampling for anti-aliasing.
 pub struct Pipeline {
     render_pipeline: wgpu::RenderPipeline,
 }
@@ -56,11 +66,11 @@ impl Pipeline {
             label: Some("FR render_pipeline"),
             layout: Some(&pipeline_layout),
             vertex: wgpu::VertexState {
-                module: &shaders.vert_122,
+                module: &shaders.vert_flat_round,
                 entry_point: "main",
                 buffers: &[wgpu::VertexBufferLayout {
                     array_stride: size_of::<Vertex>() as wgpu::BufferAddress,
-                    step_mode: wgpu::InputStepMode::Vertex,
+                    step_mode: wgpu::VertexStepMode::Vertex,
                     attributes: &wgpu::vertex_attr_array![
                         0 => Float32x2,
                         1 => Float32x4,
@@ -87,7 +97,7 @@ impl Pipeline {
                 targets: &[wgpu::ColorTargetState {
                     format: super::RENDER_TEX_FORMAT,
                     blend: Some(wgpu::BlendState::ALPHA_BLENDING),
-                    write_mask: wgpu::ColorWrite::ALL,
+                    write_mask: wgpu::ColorWrites::ALL,
                 }],
             }),
         });
@@ -130,7 +140,7 @@ impl Window {
         let na = -nb;
 
         // Since we take the mid-point, all offsets are uniform
-        let p = Vec2::splat(OFFSET / radius);
+        let p = Vec2::splat(AA_OFFSET / radius);
 
         let p1my = p1 - vy;
         let p1py = p1 + vy;
@@ -163,7 +173,7 @@ impl Window {
         ]);
     }
 
-    /// Bounds on input: `0 ≤ inner_radius ≤ 1`.
+    /// Bounds on input: `0 ≤ inner_radius ≤ 1`.
     pub fn circle(&mut self, pass: PassId, rect: Quad, inner_radius: f32, col: Rgba) {
         let aa = rect.a;
         let bb = rect.b;
@@ -173,20 +183,21 @@ impl Window {
             return;
         }
 
-        let inner = inner_radius.max(0.0).min(1.0);
+        let inner = inner_radius.clamp(0.0, 1.0);
+        let inner = inner * inner; // shader compares to square
 
         let ab = Vec2(aa.0, bb.1);
         let ba = Vec2(bb.0, aa.1);
         let mid = (aa + bb) * 0.5;
 
         let n0 = Vec2::splat(0.0);
-        let nb = (bb - aa).sign();
+        let nb = Vec2::ONE; // = (bb - aa).sign();
         let na = -nb;
         let nab = Vec2(na.0, nb.1);
         let nba = Vec2(nb.0, na.1);
 
         // Since we take the mid-point, all offsets are uniform
-        let p = nb / (bb - mid) * OFFSET;
+        let p = nb / (bb - mid) * AA_OFFSET;
 
         let aa = Vertex::new2(aa, col, inner, na, p);
         let ab = Vertex::new2(ab, col, inner, nab, p);
@@ -249,10 +260,11 @@ impl Window {
         let n0a = Vec2(0.0, na.1);
         let n0b = Vec2(0.0, nb.1);
 
-        let paa = na / (aa - cc) * OFFSET;
-        let pab = nab / (ab - cd) * OFFSET;
-        let pba = nba / (ba - dc) * OFFSET;
-        let pbb = nb / (bb - dd) * OFFSET;
+        // Size of each corner may differ, hence need for separate pixel offsets:
+        let paa = na / (aa - cc) * AA_OFFSET;
+        let pab = nab / (ab - cd) * AA_OFFSET;
+        let pba = nba / (ba - dc) * AA_OFFSET;
+        let pbb = nb / (bb - dd) * AA_OFFSET;
 
         // We must add corners separately to ensure correct interpolation of dir
         // values, hence need 16 points:
