@@ -11,6 +11,11 @@ use kas::draw::{color::Rgba, PassId};
 use kas::geom::{Quad, Vec2};
 use std::mem::size_of;
 
+/// Vertex
+///
+/// -   `screen_pos: Vec2` — screen coordinate
+/// -   `colour: Rgba`
+/// -   `dir: Vec2` — normalised direction of slope (from (-1, -1) to (1, 1))
 #[repr(C)]
 #[derive(Clone, Copy, Debug)]
 pub struct Vertex(Vec2, Rgba, Vec2);
@@ -20,6 +25,8 @@ unsafe impl bytemuck::Pod for Vertex {}
 pub type Window = common::Window<Vertex>;
 
 /// A pipeline for rendering with flat and square-corner shading
+///
+/// Does not use anti-aliasing since edges are usually pixel-aligned.
 pub struct Pipeline {
     render_pipeline: wgpu::RenderPipeline,
 }
@@ -41,11 +48,11 @@ impl Pipeline {
             label: Some("SS render_pipeline"),
             layout: Some(&pipeline_layout),
             vertex: wgpu::VertexState {
-                module: &shaders.vert_2,
+                module: &shaders.vert_shaded_square,
                 entry_point: "main",
                 buffers: &[wgpu::VertexBufferLayout {
                     array_stride: size_of::<Vertex>() as wgpu::BufferAddress,
-                    step_mode: wgpu::InputStepMode::Vertex,
+                    step_mode: wgpu::VertexStepMode::Vertex,
                     attributes: &wgpu::vertex_attr_array![0 => Float32x2, 1 => Float32x4, 2 => Float32x2],
                 }],
             },
@@ -66,7 +73,7 @@ impl Pipeline {
                 targets: &[wgpu::ColorTargetState {
                     format: super::RENDER_TEX_FORMAT,
                     blend: Some(wgpu::BlendState::ALPHA_BLENDING),
-                    write_mask: wgpu::ColorWrite::ALL,
+                    write_mask: wgpu::ColorWrites::ALL,
                 }],
             }),
         });
